#! /usr/bin/env python
'''
Module Name:    EloTouchFwFlash.py

Description:    Class for Flashing Elo Touch FW Update

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Created :  2 December 013

'''
import os



from monitorupgrade import TLog as TLOG
from monitorupgrade import Utils as utils
from monitorupgrade import Configs as Configs

import subprocess
import time

EXIT_SUCCESS = 0
MUS_LOG_FILE_DIR = '/var/log/toshiba/mus/latest/'

#Dawu Utilities    
#MXT_APP = '/opt/toshiba/mus/op_utils/mxtapp/mxt-app'
MXT_APP = '/usr/bin/mxt-app'
CONFIG_UTIL = '/opt/toshiba/mus/op_utils/configutil/configutil'
OCULAR_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/mxtapp/recovery.txt'
JIGSAW_APP = '/opt/toshiba/mus/op_utils/hlprs/extract'
MXT_APP_LOG_LEVEL = 2 

#==========================================================================================================
'''
TouchFwFlashTask - Class for Flashing Ocular/Atmel Touch FW Update
'''
#==========================================================================================================        
class TouchFwFlashTask:
    
    fw_path=''
    fw_name=''
    controller=''
    controller_count=0
    mon_list = []
    xcfg_only = False
    
    def __init__(self,path,name,controller,xcfg_only=None):
        '''
        Constructor
        '''
        print 'OcularFwFlashTask Constructed'
        TLOG.logger.info('TouchFwFlashTask Constructed')
        self.fw_path = path
        self.fw_name = name
        self.controller = controller
        self.xcfg_only = xcfg_only
    
    #This is just a workaround for the mxt-app utility not able to flash all attached monitors
    def findAttached614X(self):
        
        query_cmd = MXT_APP  + ' -q > /tmp/count' 
        process = subprocess.Popen(query_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait()                  
    
        mon_list = []
        f = open('/tmp/count', 'r+')
        lines = f.readlines()
        f.close() 
        
        process = subprocess.Popen('rm /tmp/count ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait() 
        
        for line in lines:
            line = line.strip('\n')
            usb_id, chip, vid_pid = line.split(' ')
            print usb_id, chip, vid_pid
            mon_list.append(usb_id)
        return mon_list
    
    def extractFirmware(self,path,fw_name):
        
        fw_path =  os.path.join(path,'fw')
        
        TLOG.logger.info('TouchFwFlashTask::extractFirmware() -----------------START---------------')
        dec_cmd = JIGSAW_APP + ' ' + fw_path + ' ' + fw_name + ' > /dev/null'              
        ret = utils.runShell(dec_cmd)
            
        TLOG.logger.info('TouchFwFlashTask::extractFirmware() - ret:  ' + str (ret) )
        
    def deleteFirmware(self,fw_path):
        TLOG.logger.info('TouchFwFlashTask::extractFirmware() -----------------START---------------')
        
        cmd = 'rm '   + fw_path
        
        ret = utils.runShell(cmd)   
        TLOG.logger.info('TouchFwFlashTask::deleteFirmware() - ret:  ' + str (ret) )
        
    
    def recover(self,cmd,retry=3):       
        ret = 99
        count = 0
        
        #Try to do internal recovery up to number of retries
        while(ret != 0 and count != retry):        
            txt =  'TouchFwFlashTask::run() mxt_app_cmd - RETRY#' +str(count+1) + '---\n'  
            TLOG.logger.info(txt)      
            lst = utils.find614Xin211Dmode()
            if (len(lst) != 0 ):
                for mon in lst:
                    print "[TouchFwFlashTask] - Flashing firmware to device:" + str(mon)        
                    ret = utils.runSubprocess(cmd,'MXT-APP')         
            count += 1
        return ret      
          
    def run(self):    
        TLOG.logger.info('TouchFwFlashTask::run() -----------------START---------------')
        
        self.fw_name = self.fw_name.strip('\n')
        self.extractFirmware(self.fw_path,self.fw_name)
         
        conf_path = self.fw_path.strip('\'')       
        fw_path =  os.path.join(self.fw_path,'fw')
        
        tskcfg = Configs.UpdateTaskConfig(conf_path)
        cfg_name = tskcfg.getAtmelCfgName()
        fw_name = tskcfg.getAtmelFirmwareName()
        cfg_path = (os.path.join(fw_path ,cfg_name))
        mon_list=[]
                  
        fw_file_path = (os.path.join(fw_path ,fw_name))
        TLOG.logger.info('TouchFwFlashTask::run() Locating firmware file : ' + fw_file_path)
         
       
        if utils.doesFileExist(MXT_APP) != True:                      
            print '[TouchFwFlashTask] - No MXT_APP found!'
            TLOG.logger.error('TouchFwFlashTask::run()-No MXT_APP found!')
            raise IOError('No MXT_APP script found!')
        #else:
            
        #make recovery text here
        rcv_text = 'OcularRecover:' + ',' +  fw_file_path + ',' + conf_path
        f = open(OCULAR_RECOVERY_FILE,'w')
        f.write(rcv_text)
        f.close()
            
        if ( self.xcfg_only != True ) :    
                
            print "[TouchFwFlashTask] - Starting Firmware update now..."
            print "[TouchFwFlashTask] - This will take some time,Please wait ..."
            
        
            print 'Enumerating TGCS 6149 Monitors for firmware flashing...'
            mon_list = self.findAttached614X()
                  
            #TODO: Change this once the mxt-app can support multiple monitor flashing
            if (len(mon_list) != 0 ):
                for mon in mon_list:
                        
                    #info = MXT_APP + ' -i'
                    #utils.runSubprocess(info,'MXT-APP')
                    time.sleep(20)
                    
                    print "[TouchFwFlashTask] - Flashing firmware to device:" + str(mon)
                    mxt_app_cmd = MXT_APP + ' -d ' + str(mon)  + ' --flash ' + fw_file_path + ' --verbose=' + str(MXT_APP_LOG_LEVEL)
                    print mxt_app_cmd               
                    ret = utils.runSubprocess(mxt_app_cmd,'MXT-APP')
                    
#                    lst = self.findAttached614X()
                    
                    #We try to recover here
                    if ret == 99:
                        mxt_app_cmd = MXT_APP + ' -d ' + str(mon)  + ' --flash ' + fw_file_path
                        ret = self.recover(mxt_app_cmd)
                            
                            
                    if ret != 0:
                        return ret
                    time.sleep(20)        
            else:
                    mxt_app_cmd = MXT_APP + ' --flash ' + fw_file_path
                    ret = utils.runSubprocess(mxt_app_cmd,'MXT-APP')

        #Configuration Loading
        print 'Enumerating TGCS 6149 Monitors for configuration loading...'           
        mon_list2 = self.findAttached614X()           
        if (len(mon_list2) != 0 ):
            for mon in mon_list2:
                
                #info = MXT_APP + ' -i'
                #ret = utils.runSubprocess(info,'MXT-APP')
                time.sleep(10)
                
                print "[TouchFwFlashTask] - Loading Configuration to device:" + str(mon)
                mxt_app_load_cmd = MXT_APP +  ' -d '+ str(mon) + ' --load ' + cfg_path                                            
                ret = utils.runSubprocess(mxt_app_load_cmd,'MXT-APP')
                
                if ret == 99:
                    print "[TouchFwFlashTask] - Loading configuration failed, recovering once."
                    time.sleep(10)
                    TLOG.logger.info('TouchFwFlashTask::run() mxt_app_load_cmd - RETRY---\n')
                    ret = utils.runSubprocess(mxt_app_load_cmd,'MXT-APP')
                
                    
                #FLIWA- No need to do this anymore , based on mxt-app code,they already done this.
                #Persist the config setting into the controller
#                mxt_app_bkup_cmd = MXT_APP + ' -W -T6 -r1 55 '
#                ret = utils.runSubprocess(mxt_app_bkup_cmd,'MXT-APP')
#                
#                if ret == 99:
#                    time.sleep(10)
#                    TLOG.logger.info('TouchFwFlashTask::run() mxt_app_bkup_cmd - RETRY---\n')
#                    ret = utils.runSubprocess(mxt_app_bkup_cmd,'MXT-APP')
                
                if ret != 0:
                    return ret
                time.sleep(10)

                         
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else: 
            
            #delete the the enc firmware
            self.deleteFirmware(fw_file_path)
            
            print '[TouchFwFlashTask] - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('TouchFwFlashTask::run() -----------------END-----------------\n')
            return ret
        
        
#==========================================================================================================
'''
ConfigTask - Do all stuffs here that uses Dawu Config Utility 
'''
#==========================================================================================================
class ConfigTask:
    def __init__(self):
        '''
        Constructor
        '''
        print 'Ocular ConfigTask Constructed'
        TLOG.logger.info('OcularConfigTask Constructed!')
    
    def run(self):        
        TLOG.logger.info('OcularConfigTask::run() -----------------START---------------')
        return 0
        



##==========================================================================================================
#TouchFwRecoveryFlashTask - Class for the Ocular Recovery Task for Video Flash
##==========================================================================================================        
class TouchFwRecoveryFlashTask:
    def __init__(self):
        '''
        Constructor
        '''

        TLOG.logger.info('OcularTouchFlashRecoveryTask Constructed')
    
          
    def run(self,fw_path,conf):
        TLOG.logger.info('OcularTouchFlashRecoveryTask::run()-----------------START---------------')
        
        ret = utils.doOcularRecover(fw_path,conf)
        print 'OcularTouchFlashRecoveryTask Done.' + '{' + str(ret) + '}'
        
        TLOG.logger.info('OcularTouchFlashRecoveryTask::run() - Deleting Ocular Recovery file.' )
        utils.runShell('rm ' + OCULAR_RECOVERY_FILE)
        
        TLOG.logger.info('OcularTouchFlashRecoveryTask::run()-----------------END-----------------\n')
        print 'System is rebooting now...'
        utils.runShell('reboot')
        
        #return EXIT_SUCCESS